import { proto3, Value, } from "@bufbuild/protobuf";
import { TestCase } from "./generated/tests/harness/harness_pb.js";
import fs from "node:fs";
function readTestCase(test) {
    if (test.message?.value === undefined) {
        throw new Error("Missing message");
    }
    const type = test.message.typeUrl.replace("type.googleapis.com/", "");
    const value = Value.fromBinary(test.message.value).toJsonString();
    return {
        type,
        test,
        value,
    };
}
export function readTestCases(files) {
    const cases = [];
    for (const input of files) {
        const lines = fs.readFileSync(input, "utf8").split("\n");
        for (const line of lines) {
            if (!line) {
                continue;
            }
            // Unmarshal the test case using a fake registry that simply produces a generic Value message for all messages.
            const test = TestCase.fromJson(JSON.parse(line), {
                typeRegistry: {
                    findMessage,
                },
            });
            // In absence of the full registry, simply parse the json using the generic type.googleapis.com/google.protobuf.Value.
            cases.push(readTestCase(test));
        }
    }
    return cases;
}
const registry = new Map();
function findMessage(name) {
    let type = registry.get(name);
    if (type === undefined) {
        type = class ValueFacade extends Value {
            static typeName = name;
            constructor(data) {
                super();
                proto3.util.initPartial(data, this);
            }
            static fromBinary(bytes, options) {
                return new ValueFacade().fromBinary(bytes, options);
            }
            static fromJson(jsonValue, options) {
                return new ValueFacade().fromJson(jsonValue, options);
            }
            static fromJsonString(jsonString, options) {
                return new ValueFacade().fromJsonString(jsonString, options);
            }
            static equals(a, b) {
                return proto3.util.equals(ValueFacade, a, b);
            }
        };
        registry.set(name, type);
    }
    return type;
}
